/**
 *  \file ClientSocket.cpp
 *  \brief Contains the implementation of a client-side socket wrapper.
 *
 *  Adapted from "Beej's Guide to Network Programming"
 *  http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 14:01:53 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <iostream>
#include <string>
#include "ClientSocket.h"

#define SIZE_TAG_LEN    4
#define SIZE_TAG_FORMAT "%04d"
#define BUFFER_SIZE     9999  /* = 10^SIZE_TAG_LEN - 1 */

const char* ClientSocket::dataSeparator = " ";

/**
 *  \brief Retrieve the IP address(es) of the given hostname 
 *  \param sa the address to lookup
 */
static void *get_in_addr(struct sockaddr *sa)
{
	if (sa->sa_family == AF_INET) {
		return &(((struct sockaddr_in*)sa)->sin_addr);
	}
  
	return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

ClientSocket::ClientSocket(std::string ip_addr) {  
	struct addrinfo hints, *servinfo, *p;
	int rv;
	char s[INET6_ADDRSTRLEN];
  
  // Set up the address hints structure
	memset(&hints, 0, sizeof hints);
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
  
  // Lookup the hostname or IP address
	if ((rv = getaddrinfo(ip_addr.c_str(), PORT, &hints, &servinfo)) != 0) {
		fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(rv));
		exit(1);
	}
  
	// Loop through all the results and connect to the first one we can
	for(p = servinfo; p != NULL; p = p->ai_next) {
		if ((sock = socket(p->ai_family, p->ai_socktype,
                       p->ai_protocol)) == -1) {
			perror("client: socket");
			continue;
		}
    
		if (connect(sock, p->ai_addr, p->ai_addrlen) == -1) {
			close(sock);
			perror("client: connect");
			continue;
		}
    
		break;
	}
  
	if (p == NULL) {
		fprintf(stderr, "client: failed to connect\n");
		exit(2);
	}
  
  // Log who we connected to
	inet_ntop(p->ai_family, get_in_addr((struct sockaddr *)p->ai_addr),
            s, sizeof s);
	printf("Connecting to %s\n", s);
  
  // We're done with this, so delete it
	freeaddrinfo(servinfo);
}

ClientSocket::~ClientSocket() {
  close(sock);
}

void ClientSocket::Send(Packet* data) {
  // Command -> char[] 
  char cmd[3];
  sprintf(cmd, "%d%s", data->command(), dataSeparator);
  
  // Serialise the Packet object to a space-separated string of tokens
  std::string data_string = "";
  data_string += cmd;
  data_string += data->chat()   + dataSeparator;
  if (data->message().text != "-")
    // If a Message is being sent, set the sender field to the message sender
    data_string += data->message().user + dataSeparator;
  else
    data_string += data->sender() + dataSeparator;
  data_string += data->target() + dataSeparator;
  data_string += data->message().text;
  data_string += "\0";
  
  // Set up a header indicating the length of the packet
  int packet_len = data_string.length();
  // TODO: actually handle this case - send multiple packets
  if (packet_len > BUFFER_SIZE) 
    printf("Your message is too long.\n");
  char packet_len_str[SIZE_TAG_LEN + 1];
  sprintf(packet_len_str, SIZE_TAG_FORMAT, packet_len);
  // Prepend the header to the data string
  data_string = std::string(packet_len_str) + data_string;
  
  // Write the string to the socket
  if (send(sock, data_string.c_str(), data_string.length(), 0) == -1) {
    perror("send");
  }
  
  // We're done with the Packet, so delete it
  delete data;
}

Packet* ClientSocket::Receive() {
  int  recv_size;
  int  packet_len = 0;
  char recv_buffer[BUFFER_SIZE];
  Packet* packet = NULL;
  
  // Read the header to find out how long the following data is
  switch (recv_size = recv(sock, recv_buffer, SIZE_TAG_LEN, 0)) {
    case -1:
      perror("recv");
      break;
    case 0:
      std::cerr << "Connection closed by remote party." << std::endl;
      break;
    default:
      recv_buffer[recv_size] = '\0';
      // Parse the first 
      sscanf(recv_buffer, SIZE_TAG_FORMAT, &packet_len);
      break;
  }
  
  // Read data from the socket
  switch (recv_size = recv(sock, recv_buffer, packet_len, 0)) {
    case -1:
      perror("recv");
      break;
    case 0:
      std::cerr << "Connection closed by remote party." << std::endl;
      break;
    default:
      // Ensure string is null-terminated
      recv_buffer[recv_size] = '\0';
      
      std::cout << "Received: " << recv_buffer << std::endl;
      
      // Tokenise the string and build a Packet
      char* cmd = strtok(recv_buffer, dataSeparator);
      packet = new Packet(Command(*cmd - '0'));
      packet->chat(strtok(NULL, dataSeparator));
      packet->sender(strtok(NULL, dataSeparator));
      packet->target(strtok(NULL, dataSeparator));
      Message msg(packet->sender(), strtok(NULL, ""));
      packet->message(msg);  
  } 
  
  return packet;
}

