/**
 *  \file  Fl_WindowedChat.cpp
 *  \brief Implementation of the chat GUI
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 15:38:55 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <string.h>
#include "Fl_WindowedChat.h"

/**
 *  \brief Called when the window is closed
 *  \param w a pointer to the window
 *  \param v a pointer to the observed chat
 */
static void window_callback(Fl_Widget* w, void* v) {
  static_cast<Fl_WindowedChat*>(v)->End();
  delete w;
}


Fl_WindowedChat::Fl_WindowedChat(Chat* chat, LocalUser* local_user, ChatManager* chat_manager)
: Fl_Double_Window(C_WINDOW_W, C_WINDOW_H) {
  chat_ = chat;
  local_user_ = local_user;
  chat_manager_ = chat_manager;
  
  //Set the callback for when the window is closed
  this->callback(window_callback, this);
  //Arrange the widgets in the user manager window.
  //Send button
  send = new Fl_Return_Button(C_WINDOW_W - (C_BORDER + C_SEND_W),
                              C_WINDOW_H - (C_BORDER + C_INPUT_H),
                              C_SEND_W,
                              C_INPUT_H,
                              "Send");
  send->callback(cb_send, this);
  
  //Invite button
  invite = new Fl_Button(C_WINDOW_W - (2*C_BORDER + 2*C_SEND_W),
                         C_WINDOW_H - (C_BORDER + C_INPUT_H), 
                         C_SEND_W,
                         C_INPUT_H,
                         "Invite");
  invite->callback(cb_invite, this);
  
  //Message input field
  input = new Fl_Input(C_BORDER,
                       C_WINDOW_H - (2*C_BORDER + 2*C_INPUT_H),
                       C_WINDOW_W - 2*C_BORDER,
                       C_INPUT_H,
                       "");
  
  //Message history output field
  output = new Fl_Text_Display(C_BORDER, 
                               C_BORDER,
                               C_WINDOW_W - (2*C_BORDER),
                               C_WINDOW_H - (4*C_BORDER + 2*C_INPUT_H),
                               "");
  //Output field buffer
  buffer = new Fl_Text_Buffer();
  output->buffer(buffer);
  output->wrap_mode(1, 43);
  
  //Make the window and output field resizable
  resizable(output);
  resizable(this);  
  
  end();
  show();
}

Fl_WindowedChat::~Fl_WindowedChat() { }

void Fl_WindowedChat::Update(Subject* subject) {
  Redraw();
}

/**
 *  \brief Fill the text buffer with the chat history and redraw the window.
 */
void Fl_WindowedChat::Redraw() {
  std::string text;
  //Lock the chat before reading the message history.
  Messages history = chat_->history();
  Messages::const_iterator it;
  for(it = history.begin(); it != history.end(); it++) {
    text.append((*it).user).append(": ").append((*it).text);
  }
  
  //Assign the text buffer contents, locking Fltk before
  //modifying the widget.
  Fl::lock();
  buffer->text(text.c_str());  
  //Scroll to the bottom 
  int lines = buffer->count_lines(0, buffer->length());
  output->scroll(lines, 0);
  redraw();
  Fl::awake();
  Fl::unlock();
}

/**
 *  \brief Static callback for the send button
 *  \param w a pointer to the send button
 *  \param v this
 */
inline void Fl_WindowedChat::cb_send(Fl_Widget* w, void* v) {
  static_cast<Fl_WindowedChat*>(v)->Send();
}

/**
 *  \brief Static callback for the invite button
 *  \param w a pointer to the invite button
 *  \param v this
 */
inline void Fl_WindowedChat::cb_invite(Fl_Widget* w, void* v) {
  static_cast<Fl_WindowedChat*>(v)->Invite();
}

/**
 *  \brief Callback for the send button in this instance of the class
 */
void Fl_WindowedChat::Send() {
  //Send the contents of the input field as a message. 
  Message message;
  message.user = UserID(local_user_->id());
  
  //Lock fltk before accessing and resetting the input field widget
  Fl::lock();
  message.text = std::string(input->value()).append("\n");
  input->value("");
  Fl::awake();
  Fl::unlock();
  
  chat_->Send(message);
  Redraw();
}

/**
 *  \brief Callback for the invite button in this instance of the class
 */
void Fl_WindowedChat::Invite() {
  //Pop up an input box to ask for the new recipient's ID
  Fl::lock();
  const char* invitee = fl_input("Who would you like to chat with?");
  Fl::awake();
  Fl::unlock();
  // If a UserID was entered, invite them
  if (invitee != NULL && strlen(invitee) > 0)
    chat_->Invite(local_user_->id(), invitee);
}

void Fl_WindowedChat::End() {
  chat_->Leave(local_user_->id());
  chat_manager_->EndChat(chat_->id());
}
