/**
 *  \file  Fl_WindowedUser.h
 *  \brief Declaration of the UserList GUI
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 15:38:55 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <cstdlib>
#include <cstdio>
#include "Fl_WindowedUser.h"

Fl_WindowedUser::Fl_WindowedUser(UserList* user_list, 
                                 LocalUser* local_user, 
                                 ChatManager* chat_manager)
                                :Fl_Window(UM_WINDOW_W, UM_WINDOW_H) {
  user_list_ = user_list;
  local_user_ = local_user;
  chat_manager_ = chat_manager;
                                   
  //Set the callback for when the window is closed
  this->callback(window_cb, user_list_);
  //Arrange the widgets in the user manager window.

  //Online user list output field
  users_list = new Fl_Text_Display(UM_BORDER_S,
                                   UM_BORDER_L,
                                   UM_WINDOW_W - (2*UM_BORDER_S), 
                                   UM_WINDOW_H - (UM_BORDER_L + 2*UM_BORDER_S + UM_LINE_H), 
                                   "Users");
  //Output field buffer
  users_buffer = new Fl_Text_Buffer();
  users_list->buffer(users_buffer);
  users_list->deactivate();
  users_list->show();
  
  //Username input field
  username_input = new Fl_Input(UM_BORDER_S,
                                UM_WINDOW_H - (UM_LINE_H + UM_BORDER_S),
                                UM_WINDOW_W - (3*UM_BORDER_S + UM_BUTTON_W), 
                                UM_LINE_H, 
                                "");
  
  //Login button
  login_button = new Fl_Return_Button(UM_WINDOW_W - (UM_BORDER_S + UM_BUTTON_W), 
                                      UM_WINDOW_H - (UM_BORDER_S + UM_LINE_H), 
                                      UM_BUTTON_W, 
                                      UM_LINE_H, 
                                      "Login");
  login_button->callback(login_button_cb, this);
  login_button->show();
  
  //Startchat button
  startchat_button = new Fl_Return_Button(UM_WINDOW_W - (UM_BORDER_S + UM_BUTTON_W), 
                                          UM_WINDOW_H - (UM_BORDER_S + UM_LINE_H), 
                                          UM_BUTTON_W, 
                                          UM_LINE_H, 
                                          "Start Chat");
  startchat_button->callback(startchat_button_cb, this);
  startchat_button->hide();
  
  //Make the window and the output field resizable.
  resizable(this);
  resizable(users_list);
  
  //Finish and show the user manager window.
  end();
  show();
}

Fl_WindowedUser::~Fl_WindowedUser() { }

void Fl_WindowedUser::Update(Subject* subject) {
  // Update the state of the widgets depending on the state of the UserList
  switch (local_user_->state()) {
      case LocalUser::DISCONNECTED:
        // When communication with the server has been lost
        std::printf("The server has been disconnected. Goodbye.\n");
        exit(0);
        break;
      case LocalUser::LOGGING_IN:
        // After clicking Login, before confirmation of login
        Fl::lock();
        login_button->hide();
        startchat_button->hide();
        username_input->value("");
        Fl::awake();
        Fl::unlock();
        break;
      case LocalUser::LOGGED_IN:
        // After confirmation of login
        Fl::lock();
        login_button->hide();
        startchat_button->show();
        users_list->activate();
        label(local_user_->id().c_str());
        Fl::awake();
        Fl::unlock();
        break;
      case LocalUser::LOGGED_OUT:
        // Before logging in or after unsuccessful login attempt
        Fl::lock();
        login_button->show();
        startchat_button->hide();
        users_list->deactivate();
        Fl::awake();
        Fl::unlock();
        break;
  }
  Redraw();
}
  
/**
 *  \brief Fill the text buffer with the user list and redraw the window.
 */
void Fl_WindowedUser::Redraw() {
  std::string text;
  
  //Reading the UserID list.
  Users users = user_list_->users();
  Users::const_iterator it;
  for(it = users.begin(); it != users.end(); ++it)
    text.append((*it)).append("\n");
  
  //Assign the text buffer contents, locking Fltk before modifying the widget.
  Fl::lock();
  users_buffer->text(text.c_str());
  redraw();
  Fl::awake();
  Fl::unlock();
}

/**
 *  \brief Callback for the 'Start Chat' button in this instance of the class
 */
void Fl_WindowedUser::StartChat() {
  //Read the chat target user from the input field.
  Fl::lock();
  UserID chat_target = username_input->value();
  username_input->value("");
  Fl::awake();
  Fl::unlock();
  
  if (chat_target.length() == 0) return;
  
  //Start the chat using the chat manager.
  chat_manager_->RequestChat(local_user_->id(), chat_target);
  Redraw();
}

/**
 *  \brief Callback for the 'Login' button in this instance of the class
 */
void Fl_WindowedUser::Login() {
  //Read the value of the input field and adjust
  //the visibiity of the user manager widgets, locking 
  //fltk before accessing or modifying the widgets.  
  Fl::lock();
  UserID user = username_input->value(); 
  Fl::awake();
  Fl::unlock();
  
  if (user.length() == 0) return;

  // Replace spaces with underscores
  try {
    size_t space_pos;
    while ((space_pos = user.find(" ")) != std::string::npos) 
      user.replace(space_pos, 1, "_");
  } catch (...) { }
  
  //Use the user manager to log the user in.
  local_user_->LoginAttempt(user);
  Redraw();
}

/**
 *  \brief Static callback for the window; called when closed
 *  \param w a pointer to the window
 *  \param v a pointer to the UserList
 */
void Fl_WindowedUser::window_cb(Fl_Widget* w, void* v){
  static_cast<LocalUser*>(v)->Logout();
  std::exit(0);
}

/**
 *  \brief Static callback for the 'Start Chat' button
 *  \param w a pointer to the 'Start Chat' button
 *  \param v this
 */
void Fl_WindowedUser::startchat_button_cb(Fl_Widget* w, void* v){
  static_cast<Fl_WindowedUser*>(v)->StartChat();
}

/**
 *  \brief Static callback for the 'Login' button
 *  \param w a pointer to the 'Login' button
 *  \param v this
 */
void Fl_WindowedUser::login_button_cb(Fl_Widget* w, void* v){
  static_cast<Fl_WindowedUser*>(v)->Login();
}
