/**
 *  \file Mutex.h
 *  \brief Declaration of a Mutex class which encapsulates a posix threads 
 *  mutex.
 *
 *  $LastChangedBy: jch231 $
 *  $Date: 2010-08-19 11:35:17 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef MUTEX_H
#define MUTEX_H

#include "pthread.h"

/**
 *  \class Mutex 
 *  \brief Encapsulates posix threads method calls for a mutex. 
 */
class Mutex {
public:
  /**
   *  \brief Constructor
   */
  Mutex();
  
  /**
   *  \brief Destructor
   */
  ~Mutex();
  
  /**
   *  \brief Lock the mutex. This is blocking function.
   */
  void Lock();
  
  /**
   *  \brief Unlock the mutex.
   */
  void Unlock();
  
  /**
   *  \brief Wait for the condition variable.
   */
  void Wait();
  
  /**
   *  \brief Signal the condition variable.
   */
  void Signal();
  
private:
  //The wrapped mutex
  pthread_mutex_t mutex;
  pthread_cond_t condition_variable;
};

/**
 *  \class ScopedLock 
 *  \brief Encapsulates a locking operation in the constructor of
 *  an object and the corresponding unlock in the destructor. Using a local 
 *  instance of this ScopedLock to lock a mutex will enforce the unlocking of
 *  the mutex when leaving the scope.
 */
class ScopedLock {
public:
  /**
   *  \brief Constructor. Locks mutex
   */
  ScopedLock(Mutex* mutex);
  
  /**
   *  \brief Destructor. Unlocks mutex
   */
  ~ScopedLock();
  
private:
  //Attributes
  Mutex* mutex_;
};

#endif /* MUTEX_H */
