/**
 *  \file  server/Chat.h
 *  \brief Implementation of a server-side chat
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:31:50 +1200 (Thu, 19 Aug 2010) $
 *
 */

#include <cstdio>
#include "Chat.h"

Chat::Chat(ChatID id, UserID user_sender, UserID user_target, 
           UserManager* user_manager) {
  id_ = id;
  user_manager_ = user_manager;
  mutex.Lock();
  
  //Add the sender to the chat and tell their client to start a chat.
  users_.insert(user_sender);
  user_manager_->Start(user_sender, id);
  
  //Attempt to start a chat with the target user. 
  //If they are there then add them to the chat.
  if(user_manager_->Start(user_target, id)) {
    users_.insert(user_target);
    Send(user_sender, Message(UserID("Server"), user_sender + " has invited you to chat\n"));
    Send(user_target, Message(UserID("Server"), user_target + " has joined the conversation\n"));
  }
  else
    Send(user_target, Message(UserID("Server"), user_target + " is not avaliable to chat\n"));
  
  mutex.Unlock();
}

void Chat::Receive(Message message) {
  mutex.Lock();
  Send(message.user, message);
  mutex.Unlock();
}

void Chat::Logout(UserID user) {
  // If the user is in this chat, remove them
  if(users_.find(user) != users_.end())
    Leave(user);
}

void Chat::Leave(UserID user) {
  mutex.Lock();
  users_.erase(user);
  Send(user, Message(UserID("Server"), user + " has left the conversation\n"));
  mutex.Unlock();
}

void Chat::Invite(UserID user_sender, UserID user_target) {
  mutex.Lock();
  // If the user is not in this chat already
  if(users_.find(user_target) == users_.end()) {
    // Tell them to start a new chat
    if(user_manager_->Start(user_target, id_)) {
      // Add them to this chat
      users_.insert(user_target);
      user_manager_->Send(user_target, id_, Message(UserID("Server"), user_sender + " has invited you to chat\n"));
      Send(user_target, Message(UserID("Server"), user_sender + " invited " + user_target + " to the chat\n"));
    }
    else
      Send(user_target, Message(UserID("Server"), user_sender + 
                                                  " invited " + 
                                                  user_target + 
                                                  ", but " +
                                                  user_target + 
                                                  " is not available to chat\n"));
  }
  mutex.Unlock();
}

void Chat::Send(UserID user, Message message) {
  Users::iterator it;
  // For each user in this chat
  for(it = users_.begin(); it != users_.end();) {
    // Except the one that sent the message
    if((*it) != user) {
      // Send the message to them
      if(!user_manager_->Send((*it), id_, message)) {
        // Remove them from the chat if sending failed
        UserID absent_user = (*it);
        users_.erase(it++);
        Send(absent_user, Message(UserID("Server"), absent_user + " is not available to chat\n"));
      }
      else
        ++it;
    }
    else
      ++it;
  }
  
}