/**
 *  \file  ChatManager.h
 *  \brief Declaration of a chat manager
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:20:28 +1200 (Thu, 19 Aug 2010) $
 *
 */

#ifndef CHATMANGER_H
#define CHATMANGER_H

#include "TypeDef.h"
#include "Mutex.h"

#include "Chat.h"
#include "UserManager.h"
#include <map>

/** Map ChatIDs to Chat objects */
typedef std::map<ChatID, Chat*> ChatMap;

/**
 *  \class ChatManager
 *  \brief A class to manage chats and their interactions
 */
class ChatManager {
public:
  /**
   *  \brief Constructor
   *  \param user_manager the UserManager to use
   */
  ChatManager(UserManager* user_manager);
  
  /**
   *  \brief Destructor
   */
  ChatManager(){}
  
  /**
   *  \brief Start a chat
   *  \param user_sender the user initiating the chat
   *  \param user_target the user they want to chat with
   */
  void Start(UserID user_sender, UserID user_target);
  
  /**
   *  \brief Invite another user to an existing chat
   *  \param chat the chat to work with
   *  \param user_sender the user initiating the invitation
   *  \param user_target the invited user
   */
  void Invite(ChatID chat, UserID user_sender, UserID user_target);
  
  /**
   *  \brief Remove a user from a chat
   *  \param chat the chat to remove the user from
   *  \param user the user to remove
   */
  void Leave(ChatID chat, UserID user);
  
  /**
   *  \brief Log a user out
   *  \param user the user to log out
   */
  void Logout(UserID user);
  
  /**
   *  \brief Send a message to a chat
   *  \param chat the chat to send the message to
   *  \param message the message to send
   */
  void Send(ChatID chat, Message message);
    
private:
  
  //Attributes
  ChatMap chats_;
  UserManager* user_manager_;
  int chat_id_count;
  
  //Thread safety
  Mutex mutex;
};

#endif /* CHATMANAGER_H */

