/**
 *  \file server/UserManager.h
 *  \brief Declares UserManager for server
 *
 *  $LastChangedBy: sjk114 $
 *  $Date: 2010-08-19 16:42:25 +1200 (Thu, 19 Aug 2010) $
 *  
 */

#ifndef USERMANAGER_H
#define USERMANAGER_H

#include "TypeDef.h"
#include "Observable.h"
#include "Mutex.h"

#include "Outbox.h"
#include <map>

/** 
 *  \typedef UserMap 
 *  \brief A map of user ids to outboxes
 */
typedef std::map<UserID, Outbox*> UserMap;

/**
 *  \class UserManager
 *  \brief Handles the record of users online. UserManager uses a mutex to 
 *  provided synchronised access to the list of users.
 *
 *  Checks user login attempts against the user list. Any users being added or 
 *  removed are pushed out to the clients. Also used by Chats to access their 
 *  participant user's outboxes in order to send messages.
 */
class UserManager {
public:
  /**
   *  \brief Constructor
   */
  UserManager();
  
  /**
   *  \brief Destructor
   */
  ~UserManager(){}

  /**
   *  \brief Log a user in.
   *  \param user The user to add.
   *  \param outbox The connection to link the user too.
   */
  void Login(Outbox* outbox, UserID user);
  
  /**
   *  \brief Logout a user.
   *  \param user The user to logout.
   */
  void Logout(UserID user);
  
  /**
   *  \brief Send a message to a user.
   *  \param user The user to send the message to.
   *  \param chat The chat that the message is part of.
   *  \param message The message to send.
   *  \return false If the user is not logged in.
   */
  bool Send(UserID user, ChatID chat, Message message);
  
  /**
   *  \brief Tell a user to start a chat.
   *  \param user The user to tell to start the chat.
   *  \param chat The id of the chat to start.
   */
  bool Start(UserID user, ChatID chat);
  
private:
  
  //Attributes
  UserMap users_;
  
  //Thread control
  Mutex mutex;
};

#endif /* USERMANAGER_H */

